# coding: utf-8

import datetime

from aio_client.base import RequestTypeEnum
from aio_client.base import _prepare_log
from aio_client.base import send_request
from aio_client.base.helpers import create_message_id
from aio_client.base.helpers import post_request
from aio_client.base.signals import get_data_done
from aio_client.base.signals import robust_sender

from .const import CHANGE_MESSAGE_ID_IN_DAYS
from .models import GetConsumerReceipt
from .models import GetConsumerResponse
from .models import PostConsumerRequest


@robust_sender(get_data_done, sender=GetConsumerResponse)
def consumer_get_requests():
    """Получение всех ответов из очереди СМЭВ
    :return list список словарей с параметрами ответа
    """
    request_log = _prepare_log(RequestTypeEnum.CS_GET)

    response = send_request(request_log)
    for message in response.json():
        message['request_id'] = request_log
        GetConsumerResponse.objects.create(**message)
    return response.json()


def consumer_post_request(request_msg):
    """Передача заявок в СМЭВ

    Если сообщение отправляется повторно, то создаем message_id заново
    :param request_msg: объект класса PostConsumerRequest
     c параметрами ответа
    :return: инстанс класса requests.models.Response, либо None
    """
    assert isinstance(request_msg, PostConsumerRequest)
    if hasattr(request_msg, 'request_id'):
        diff_in_days = abs(
            request_msg.request_id.timestamp_created - datetime.datetime.now()
        ).days
        if diff_in_days >= CHANGE_MESSAGE_ID_IN_DAYS:
            request_msg.message_id = create_message_id()
    return post_request(request_msg)


@robust_sender(get_data_done, sender=GetConsumerReceipt)
def consumer_get_receipt():
    """Получение ответа СМЭВ по всем отправленным заявкам
    :return dict словарь с параметрами ответа
    """
    request_log = _prepare_log(RequestTypeEnum.CS_GET_R)
    response = send_request(request_log)
    for message in response.json():
        message.update(
            request_id=request_log,
            state=GetConsumerReceipt.SENT
        )

        GetConsumerReceipt.objects.create(**message)
    return response.json()
